// Copyright (2013) Jann Horn <jann@thejh.net>

// Supports very simple ini files. Structure:
//  - data is in lines, key=value
//  - # or ; starts a comment line
//  - lines without = are ignored, too

HEADER #include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

HEADER struct inifile_entry {
HEADER   char *key;
HEADER   char *value;
HEADER   struct inifile_entry *next;
HEADER };
HEADER 
HEADER struct inifile {
HEADER   struct inifile_entry *first;
HEADER };

// Parses an inifile. When you're done, you can free the whole thing
// (including all the entries and the strings) by freeing the pointer
// that this function returns.
PUBLIC_FN struct inifile *parse_inifile(char *f) {
  int n_of_lines = count_char_occurences(f, '\n')+1;
  size_t worst_case_size = strlen(f) +
                           n_of_lines*sizeof(struct inifile_entry) +
                           sizeof(struct inifile);
  
  char *buf = malloc(worst_case_size);
  if (buf == NULL) return NULL;
  
  struct inifile *head = (void*)buf;
  void *buf_ = (void*)(head+1);
  
  struct inifile_entry **nextp = &head->first;
  
  while (*f) {
    char *nl = strchr(f, '\n');
    if (!nl) nl=f+strlen(f);
    if (*f == '#' || *f == ';') goto next;
    char *eqsign = strchr(f, '=');
    if (!eqsign) break;
    if (eqsign > nl) goto next;
    
    struct inifile_entry *e = buf_; buf_ = e+1;
    *nextp = e; nextp = &e->next;
    char *key = f;
    int keylen = eqsign-key;
    char *value = eqsign+1;
    int valuelen = nl-value;
    e->key = buf_; buf_+=keylen+1;
    memcpyn(e->key, key, keylen);
    trim_end(e->key, " \r");
    e->value = buf_; buf_+=valuelen+1;
    memcpyn(e->value, value, valuelen);
    trim_end(e->value, " \r");
    
next:
    f = nl;
    if (!*f) break;
    f++;
  }
  
  *nextp = NULL;
  return head;
}

PUBLIC_FN struct inifile *fslurp_inifile(int fd) {
  char *f = slurp_fd(fd, NULL, 0);
  if (f == NULL) return NULL;
  struct inifile *r = parse_inifile(f);
  int errno_ = errno;
  free(f);
  errno = errno_;
  return r;
}

PUBLIC_FN struct inifile *slurp_inifile(char *path) {
  int fd = open(path, O_RDONLY);
  if (fd == -1) return NULL;
  struct inifile *r = fslurp_inifile(fd);
  int errno_ = errno;
  close(fd);
  errno = errno_;
  return r;
}

PUBLIC_FN char *inifile_lookup(struct inifile *inf, char *key) {
  for (struct inifile_entry *e = inf->first; e; e=e->next) {
    if (streq(e->key, key)) return e->value;
  }
  return NULL;
}
